`timescale 1ns / 1ps

////////////////////////////////////////////////////////////////////////////////
// Company:       UPB
// Engineer:      Dan Dragomir
//
// Create Date:   13:05:30 09/11/2017
// Design Name:   tester combinational
// Module Name:   tester
// Project Name:  practical test
// Target Device: ISim
// Tool versions: 14.6
// Description:   tester for combinational module
////////////////////////////////////////////////////////////////////////////////

module tester;

`include "adapter.inc"

parameter early_exit = 0;                   // boolean; bail on first error
parameter show_output = 0;                  // boolean; show what is being tested
parameter max_errors = 32;                  // integer; maximum number of errors to show

// UUT and reference
wire [`OUTPUT_WIDTH-1 : 0] tst_out, ref_out;
reg  [`INPUT_WIDTH-1 : 0] in;
adapter adapter (
    .out_uut(tst_out),
    .out_ref(ref_out),
    .in(in)
);

// Tester
integer i, j;                               // counters that enumerate all input operands

integer results_good;                       // number of outputs computed correctly
integer results_total;                      // total number of outputs

real percentage;                            // test passed percentage
real grade;                                 // test grade
integer file;                               // results file

initial begin
    results_good = 0;
    results_total = 0;
    for(i = 0; i < 2 ** `INPUT_WIDTH; i = i + 1) begin
        results_total = results_total + 1;

        if(show_output)
            print_input(in);

        in = i;
        #1;                                 // needed to force update of output signals

        if(tst_out === ref_out) begin       // result is good
            results_good = results_good + 1;

            if(show_output) begin
                $write(", ");
                print_output(ref_out);
                $write("\tok\n");
            end
        end
        else begin                          // result is wrong
            if(show_output)
                $write("\n");

            if(show_output || (results_total - results_good <= max_errors)) begin
                $write("\terror: ");
                print_input(in);
                $write(", ");
                print_output(tst_out);
                $write(", expected ");
                print_output(ref_out);
                $write("\n");
            end
            if(!show_output && (results_total - results_good == max_errors)) begin
                $write("\t.\n");
                $write("\t.\n");
                $write("\t.\n");
            end

            if(early_exit) begin            // force exit from testing loop
                i = 2 ** `INPUT_WIDTH;
            end
        end
     end

    if(results_good == results_total)
        $write("test ok\n");

    percentage = results_good * 1.0 / results_total;
    grade = results_good * 1.0 / results_total;

    // print results
    file = $fopen("result.tester");
    $fwrite(file, "%6.2f: %0d correct out of %0d (%6.2f%% completed)\n", 10.0 * (grade - 1.0), results_good, results_total, 100.0 * percentage);
    $fclose(file);
end

endmodule
